"""
-[ ] Option to record the all the cpu freqs
-[ ] Do some analysis on them based on time and maybe what was running from processes
"""

import os
import time
import psutil
import getpass
import main.tools

from charts import HorizontalBarChart, Chart, Options


class CPUFreq:

    _max = psutil.cpu_freq().max
    _min = psutil.cpu_freq().min

    @property
    def max_freq(self) -> int:
        return self._max
    
    @max_freq.setter
    def max_freq(self, maximum: int) -> None:
        self._max = maximum

    @property
    def min_freq(self) -> int:
        return self._min

    @min_freq.setter
    def min_freq(self, minimum: int) -> None:
        self._min = minimum

    def __init__(self) -> None:
        return

    def display_separately(self, save=True) -> None:
        os.system('cls' if os.name == 'nt' else 'clear')
        ch = HorizontalBarChart()
        ch.options.graph_color = 'white'

        while True:
            cpu = psutil.cpu_freq(percpu=True)
            
            for n, i in enumerate(cpu, start=1):
                percent = round((i.current-self._min) / (self._max-self._min) * 100, 2)
                ch.chart(
                    title=f'CPU #{n}',
                    pre_graph_text=f'Current: {round(i.current, 1)}hz || Min: {self._min}hz || Max: {self._max}hz',
                    post_graph_text=tools.create_usage_warning(percent, 30, 15),
                    footer=None,
                    maximum=self._max - self._min,
                    current=i.current - self._min
                )

                if save:
                    cpu = {
                        'user': [getpass.getuser()],
                        'cpu': [n],
                        'time': [time.time()],
                        'current': [i.current],
                        'usage': [percent]
                        }

                    tools.save_to_csv(cpu, '~/cpus.csv')

                print()

            time.sleep(0.8)
            os.system('cls' if os.name == 'nt' else 'clear')
        tools.save_to_csv(cpu, '~/cpu.csv')
    
    def display_combined(self) -> None:
        os.system('cls' if os.name == 'nt' else 'clear')
        ch = HorizontalBarChart()
        ch.options.graph_color = 'white'
        while True:
            cpu = psutil.cpu_freq(percpu=False)
            ch.chart(
                title='CPU (ALL)',
                pre_graph_text=f'Current: {round(cpu.current, 1)}hz || Min: {self._min}hz || Max: {self._max}hz',
                post_graph_text=tools.create_usage_warning(
                                round((cpu.current-self._min) / (self._max-self._min) * 100, 2),
                                30, 15),
                footer=None,
                maximum=self._max - self._min,
                current=cpu.current - self._min
            )
            print()

            time.sleep(0.8)
            os.system('cls' if os.name == 'nt' else 'clear')


if __name__ == '__main__':
    cpu_freq = CPUFreq()
    cpu_freq.display_separately()


# Battery module for vizex

import datetime
import psutil

from math import ceil
from charts import Chart, HorizontalBarChart, Options


class Battery:
    """Personalize and visualize the Battery usage in the terminal"""

    def __init__(self) -> None:
        """ Create a new Battery Object """
        self._battery = psutil.sensors_battery()
        if not self._battery:
            raise Exception("Battery information currently unavailable")

    def print_charts(self, options: Options=None) -> None:
        """ Prints battery information """
        if not options: options = Options()
        chart = HorizontalBarChart(options)
        self.print_battery_chart(chart)

    def print_battery_chart(self, chart: Chart) -> None:
        """ Prints battery information chart """
        post_graph_text = str(
                        ceil(100 * self._battery.percent) / 100) + "%"
        footer = self.create_details_text()

        chart.chart(
            post_graph_text=post_graph_text,
            pre_graph_text=None,
            title="Battery",
            footer=footer,
            maximum=100,
            current=self._battery.percent,
        )
        print()

    def create_details_text(self) -> str:
        """ Format more information about the battery """
        time_left = datetime.timedelta(seconds=self._battery.secsleft)

        if (plugged := self._battery.power_plugged):
            ret = format("Charging")
        else:
            ret = f"Plugged in: {plugged}\tDischarging: {time_left}\t"
        return ret

if __name__ == "__main__":
    battery = Battery()
    battery.print_charts()
    # Example output:

    # Battery
    # █████████████▒░░░░░░░░░░░░░░░░░░░░░░░░░ 34.23%
    # Charging

    # Battery
    # █████████████▒░░░░░░░░░░░░░░░░░░░░░░░░░ 34.51%
    # Plugged in: False       Discharging: 0:48:24



# Command line interface for VIZEX and VIZEXdf

import click
import sys

from disks import DiskUsage
from files import DirectoryFiles
from battery import Battery
from charts import Options
from cpu import CPUFreq
from tools import append_to_bash
from colored import fg, attr, stylize

# ----- vizexdf options and arguments -----
@click.version_option('1.5.5', message='%(prog)s version %(version)s')
@click.command(options_metavar='[options]')
@click.argument(
    'path', 
    type=click.Path(exists=True),
    default='.',
    metavar='[path]'
)
@click.option(
    '-s', '--sort',
    type=click.Choice(['type', 'size', 'name', 'dt']),
    default='type',
    help='Sort table with one of four given columns'
)
@click.option(
    '-a', '--all',
    is_flag=True,
    help='Include hidden files and folders'
)
@click.option(
    '-d', '--desc',
    is_flag=True,
    help='Sort columns in descending order')
@click.option(
    '-l', '--alias',
    is_flag=True,
    help='Store customized terminal command for vizexdf as an alias so you don\'t have to repeat the line everytime.'
        + '<-l> should always be the last command in the line'
)
def dirs_files(sort: str, all: str,
            desc: str, path: str, alias: str) -> None:
    '''
\b
██╗   ██╗██╗███████╗███████╗██╗  ██╗     _  __ 
██║   ██║██║╚══███╔╝██╔════╝╚██╗██╔╝  __| |/ _|
██║   ██║██║  ███╔╝ █████╗   ╚███╔╝  / _` | |_ 
╚██╗ ██╔╝██║ ███╔╝  ██╔══╝   ██╔██╗ | (_| |  _|
 ╚████╔╝ ██║███████╗███████╗██╔╝ ██╗ \__,_|_|  
  ╚═══╝  ╚═╝╚══════╝╚══════╝╚═╝  ╚═╝
    Made by: Beka Modebadze

The command-line program `vizexdf` is a branch of the `vizex` that 
displays directories and files information in a tabular form.

`vizexdf` Prints data of current working path in a tabular form.
You can pass a path for a specific directory you want to print.

    Example: vizexdf /home/bexx/test

You can also chain options for --all --desc --sort.

    Example: vizexdf -ads name

This will sort in descending order by name and show all the hidden files and folders.
!Just make sure 's' is placed at the end of the options chain!
    '''
    if alias: # Set vizexdf as alias
        line = 'vizexdf ' + ' '.join(sys.argv[1:-1])
        append_to_bash('vizexdf', line)
    show = all
    desc_sort = desc
    sort_by = sort
    dirpath = path

    # Execute vizexdf
    dir_files = DirectoryFiles(sort_by=sort_by, show_hidden=show,
                                desc=desc_sort, dpath=dirpath)
    dir_files.print_tables()


# ----- vizex options and arguments -----
@click.version_option('1.5.5', message='%(prog)s version %(version)s')
@click.command(options_metavar='[options]')
@click.argument('arg',
                default='disk',
                metavar='command')
@click.option(
    "--save",
    help="Export your disk usage data into a CSV or JSON file:"
        + "Takes a full path with a file name as an argument. "
        + "File type will be defined based on a <.type> of the filename"
)
@click.option(
    "-P",
    "--path",
    default=None,
    multiple=True,
    help="Print directory for a provided path."
        + " It can be both, full and relative path",
)
@click.option(
    "-X",
    "--exclude",
    default=None,
    multiple=True,
    help="Select partition you want to exclude",
)
@click.option(
    "--every",
    is_flag=True,
    help="Display information for all the disks"
)
@click.option(
    "--details",
    is_flag=True,
    help="Display additinal details like fstype and mountpoint",
)
@click.option(
    "-d",
    "--header",
    default=None,
    type=str,
    metavar="[COLOR]",
    help="Set the partition name color",
)
@click.option(
    "-s",
    "--style",
    default=None,
    type=str,
    metavar="[ATTR]",
    help="Change the style of the header's display",
)
@click.option(
    "-t",
    "--text",
    default=None,
    type=str,
    metavar="[COLOR]",
    help="Set the color of the regular text",
)
@click.option(
    "-g",
    "--graph",
    default=None,
    type=str,
    metavar="[COLOR]",
    help="Change the color of the bar graph",
)
@click.option(
    "-m",
    "--mark",
    default=None,
    help="Choose the symbols used for the graph"
)
@click.option(
    '-l', '--alias',
    is_flag=True,
    help='Store customized terminal command for vizexdf as an alias so you don\'t have to repeat the line everytime.'
        + '<-l> should always be the last command in the line'
)
def disk_usage(arg, save, path, every,
        details, exclude, header, style,
        text, graph, mark, alias) -> None:
    """
\b                                            
██╗   ██╗██╗███████╗███████╗██╗  ██╗
██║   ██║██║╚══███╔╝██╔════╝╚██╗██╔╝
██║   ██║██║  ███╔╝ █████╗   ╚███╔╝ 
╚██╗ ██╔╝██║ ███╔╝  ██╔══╝   ██╔██╗ 
 ╚████╔╝ ██║███████╗███████╗██╔╝ ██╗
  ╚═══╝  ╚═╝╚══════╝╚══════╝╚═╝  ╚═╝
    Made by: Beka Modebadze

<< Customize and display Disk Usage in the terminal >>

\b
COLORS: light_red, red, dark_red, dark_blue, blue,
    cyan, yellow, green, pink, white, black, purple,
    neon, grey, beige, orange, magenta, peach.

\b
ATTRIBUTES: bold, dim, underlined, blink, reverse.

\b
You can also give *args like [BATTERY] and [CPU]

\b
battery --> will display the battery information if found.
cpu --> will visualize the usage of each CPU in live time *(beta mode)
    """
    if alias: # Set vizex as alias
        line = 'vizex ' + ' '.join(sys.argv[1:-1])
        append_to_bash('vizex', line)

    options: Options = Options()

    if mark:
        options.symbol = mark
    if header:
        options.header_color = header
    if text:
        options.text_color = text
    if graph:
        options.graph_color = graph
    if style:
        options.header_style = style
    exclude_list = list(exclude)

    if arg == 'battery':
        try:
            battery = Battery()
            battery.print_charts()
        except Exception:
            print('Battery not found!')
    elif arg == 'cpu':
        cpus = CPUFreq()
        cpus.display_separately()
    else:
        renderer = DiskUsage(
            path=path, exclude=exclude_list, details=details, every=every
        )
        if save:
            renderer.save_data(save)
        renderer.print_charts(options)


if __name__ == "__main__":
    dirs_files()



# Class to retrieve, organize, and print disk usage/disk space data

import psutil
import platform
import pandas as pd

from math import ceil
from colored import fg, attr, stylize
from charts import Chart, HorizontalBarChart, Options
from tools import create_usage_warning, ints_to_human_readable
from tools import save_to_csv, save_to_json, create_usage_warning


class DiskUsage:
    """
    Personalize and visualize the disk space usage in the terminal
    """

    def __init__(self,
                path: str = "",
                exclude: list=None,
                details: bool=False,
                every: bool=False) -> None:
        self.path = path
        self.exclude = []
        if exclude:
            self.exclude = exclude
        self.details = details
        self.every = every
        self._platform = platform.system() # Check on which platform vizex operates

    def print_charts(self, options: Options=None) -> None:
        """
        Prints the charts based on user selection type
        
        Args:
            options (Options): colors and symbols for printing
        """
        if not options:
            options = Options()

        if self.path:
            parts = self.grab_specific(self.path[0])
        else:
            parts = self.grab_partitions(exclude=self.exclude,
                                        every=self.every)

        chrt = HorizontalBarChart(options)
        for partname in parts:
            self.print_disk_chart(chrt, partname, parts[partname])

    def print_disk_chart(
            self, ch: Chart, partname: str, part: dict
        ) -> None:
        """Prints the disk data as a chart
        
        Args:
            ch (Chart): to print
            partname (str): partition title
            part (dict): parition data to be visualized
        """
        pre_graph_text = self.create_stats(part)

        footer = None
        if self.details:
            footer = self.create_details_text(part)

        maximum = part["total"]
        current = part["used"]
        post_graph_text = create_usage_warning(
                                part['percent'], 80, 60)

        ch.chart(
            post_graph_text=post_graph_text,
            title=partname,
            pre_graph_text=pre_graph_text,
            footer=footer,
            maximum=maximum,
            current=current,
        )
        print()

    def grab_root(self) -> dict:
        """
        Grab the data for the root partition
        
        return:
            (dict) with column titles as keys
        """
        return {
            "total": psutil.disk_usage("/").total,
            "used": psutil.disk_usage("/").used,
            "free": psutil.disk_usage("/").free,
            "percent": psutil.disk_usage("/").percent,
            "fstype": psutil.disk_partitions(all=False)[0][2],
            "mountpoint": "/",
        }

    def grab_partitions(self,
                        exclude: list,
                        every: bool) -> dict:
        """Grabs all the partitions data
        
        Args:
            exclude (list): of partitions to exclude
            every (bool): if all the partitions should be grabbed
        """
        disks = {}
        
        # If we don't need every part we grab root seperately
        if not every and self._platform != 'Windows':
            disks['root'] = self.grab_root()
        disk_parts = psutil.disk_partitions(all=every)
        
        for disk in disk_parts[1:]:

            # Exclude mounpoints created by snap
            if disk.device.startswith("/dev/loop"):
                continue

            # Check that tmp is not slipping as partition
            if disk.mountpoint.startswith("/tmp"):
                continue

            # Check that part name is not in the excluded list
            if self.exclude and disk[1].split("/")[-1] in exclude:
                continue

            try:
                if psutil.disk_usage(disk[1]).total > 0:
                    disks[disk[1].split("/")[-1]] = {
                        "total": psutil.disk_usage(disk[1]).total,
                        "used": psutil.disk_usage(disk[1]).used,
                        "free": psutil.disk_usage(disk[1]).free,
                        "percent": psutil.disk_usage(disk[1]).percent,
                        "fstype": disk.fstype,
                        "mountpoint": disk.mountpoint,
                    }
            except Exception as e:
                print(e)
                continue
        return disks

    def grab_specific(self, disk_path: str) -> dict:
        """
        Grabs data for the partition of the user specified path

        Args:
            disk_path (str): to the partition to grab
        """
        disks = {}
        disks[disk_path] = {
            "total": psutil.disk_usage(disk_path).total,
            "used": psutil.disk_usage(disk_path).used,
            "free": psutil.disk_usage(disk_path).free,
            "percent": psutil.disk_usage(disk_path).percent,
            "fstype": "N/A",
            "mountpoint": "N/A",
        }
        return disks

    def create_details_text(self, disk: dict) -> str:
        """
        Creates a string representation of a disk
        
        Args:
            disk (dict): text to print
        """
        return f"fstype={disk['fstype']}\tmountpoint={disk['mountpoint']}"

    def create_stats(self, disk: dict) -> str:
        """
        Creates statistics as string for a disk
        
        Args:
            disk (dict): stats to print
        """
        r = ints_to_human_readable(disk)
        return f"Total: {r['total']}\t Used: {r['used']}\t Free: {r['free']}"

    def save_data(self, filename: str) -> None:
        """
        Outputs disks/partitions data as a CSV file
        
        Args:
            filename (str): for the saved file
        """
        data = self.grab_partitions(self.exclude, self.every)
        if (file_type := filename.split(".")[-1].lower()) == 'csv':
            save_to_csv(data, filename)
        elif file_type == 'json':
            save_to_json(data, filename)
        else:
            raise NameError("Not supported file type, please indicate "
                            + ".CSV or .JSON at the end of the filename")


if __name__ == "__main__":
    self = DiskUsage()
    parts = self.grab_partitions(exclude=[], every=False)

    for partname in parts:
        part = parts[partname]
        ch = HorizontalBarChart()
        title = (partname,)
        pre_graph_text = self.create_stats(part)
        footer = self.create_details_text(part)
        maximum = part["total"]
        current = part["used"]
        post_graph_text = create_usage_warning(
                                part['percent'], 80, 60)

        ch.chart(
            post_graph_text=post_graph_text,
            title=title[0],
            pre_graph_text=pre_graph_text,
            footer=footer,
            maximum=maximum,
            current=current,
        )
        print()



# Utility functions and Classes for vizex/vizexdf

import os
import json
import time
import pandas as pd

from math import ceil
from dataclasses import dataclass
from colored import fg, attr, stylize


def bytes_to_human_readable(bytes: int, suffix='b') -> str:
    """
    Converts bytes into the appropriate human
    readable unit with a relevant suffix.

    Args:
        bytes (int): to convert
        suffix (str, optional): suffix of a size string.
                                Defaults to 'b'.

    Returns:
        str: [description]
    """
    for unit in ['','k','m','g','t','p','e','z']:
        if abs(bytes) < 1024.0:
            return f'{bytes:3.1f} {unit}{suffix}'
        bytes /= 1024.0
    return f'{bytes:.1f} {"Y"}{suffix}'

def ints_to_human_readable(disk: dict) -> dict:
    """Converts the dictionary of integers
    into the human readable size formats.

    Args:
        disk [dict]: of large byte numbers

    Returns:
        dict: same dict but in human readable format
    """
    result = {}
    try:
        for key in disk:
            result[key] = bytes_to_human_readable(disk[key])
    except Exception:
        result[key] = disk[key] # If not able to convert return input back
    return result

def printml(folder: list, cols: int = 1) -> None:
    """Prints multiline strings side by side.

    Args:
        folder (list): list of folders to print
        cols (int, optional): On how many lines should it be printed.
                            Defaults to 1.
    """
    size = len(folder)
    incr = ceil(size / cols)
    end, start = 0, 0
    while True:
        if end >= size:
            break
        end += incr
        # Check if the end index exceeds the last index
        if end > size:
            end = size
        lines = [folder[i].splitlines() for i in range(start, end)]
        for line in zip(*lines):
            print(*line, sep='  ')
        print()
        start = end

def create_usage_warning(usage_pct: float,
                        red_flag: int,
                        yellow_flag: int) -> str:
    """Create disk usage percent with warning color

    Args:
        usage_pct (float): of a given space
        red_flag (int): threshold that decides if print should be red
        yellow_flag (int): threshold that decides if print is yellow

    Returns:
        str: [description]
    """
    if usage_pct < 0:
        usage_pct = 0

    if usage_pct > 100:
        usage_pct = 100

    use = str(usage_pct) + '% used'

    if usage_pct >= red_flag:
        return f"{stylize(use, attr('blink') + fg(9))}"
    elif usage_pct >= yellow_flag:
        return f"{stylize(use, fg(214))}"
    else:
        return f"{stylize(use, fg(82))}"

def save_to_csv(data: dict,
                filename: str,
                orient: str='index') -> None:
    """Outputs disks/partitions data as a CSV file

    Args:
        data (dict): to be saved to a CSV file
        filename (str): to name a saved file
        orient (str, optional): how lines are saved.
                                Defaults to 'index'.

    Raises:
        NameError: [description]
    """
    if filename.split(".")[-1].lower() == 'csv':
        df = pd.DataFrame.from_dict(data, orient=orient)
        df.to_csv(filename, mode='a')
    else:
        raise NameError('Please include ".csv" in the filename')

def save_to_json(data: dict,
                filename: str,
                indent: int=4) -> None:
    """Saves disk/partitions data as a JSON file

    Args:
        data (dict): to be saved to a JSON file
        filename (str): to name a saved file
        indent (int, optional): of each new line.
                                Defaults to 4.

    Raises:
        NameError: [description]
    """
    if filename.split(".")[-1].lower() == 'json':
        with open(filename, "w") as f:
            json.dump(data, f, indent=indent)
    else:
        raise NameError('Please include ".json" in the filename')

def append_to_bash(alias: str, line: str) -> None:
    """
    Appends terminal command line as an alias in .bashrc for reuse

    Args:
        alias[str]: To set up in the bash
        line[str]: line which will be assigned to an alias
    """
    bash = os.path.expanduser("~") + '/.bash_aliases'
    print(remove_if_exists(alias, bash))
    with open(bash, 'a+') as f:
            # if line.startswith(f'alias {alias}')
        f.write('alias ' + alias + f"='{line}'")

def remove_if_exists(alias: str, path: str) -> None:
    """
    Removes if the given line/alias exists in a given file

    Args:
        alias (str): to check if exists in bash
        path (str): path to the file to read
    """
    if not os.path.exists(path):
        return
    with open(path, "r") as f:
        lines = f.readlines()
    with open(path, "w") as f:
        for line in lines:
            # We only write back lines which is not alias
            if f'alias {alias}' not in line.strip("\n"):
                f.write(line)

def normalize_date(format: str, date: int) -> str:
    """
    Converts date from nanoseconds to the fuman readable form

    Args:
        format (str): example %h-%d-%Y for mm-dd-yyyy
        date_as_nanosecs (int): date in nanoseconds

    Returns:
        str: Human readable format of a date
    """
    return time.strftime(format, time.localtime(date))


@dataclass(order=True)
class DecoratedData():
    """
    Custom class to compare numerical data for sorting
    which appears in the stylized representation of a string.
    """
    size: int
    to_string: str

    def __str__(self):
        """String representation of the class"""
        return self.to_string


if __name__ == '__main__':
    file1 = DecoratedData(55456, '54.2 kb')
    file2 = DecoratedData(123233419, '117.5 mb')
    print(f'{file1} is less than {file2} : {file1 < file2}')


# Class to collect and organize data about files and directories

import os
import getpass
import time
import magic

from tabulate import tabulate
from colored import fg, bg, stylize, attr
from tools import DecoratedData, bytes_to_human_readable,  normalize_date

class DirectoryFiles():
    """
    Creates the tabular listing of all the folders and files in a given path.
    This module can be seen as a substitute for du/df Linux terminal commands.

    Args:
        path (str): to the file or directory
        shoiw_hidden (bool): if program'd show hidden files and folders
        sort_by (str): at which column folders and files should be sorted
        desc (bool): if the sorting should be descending or ascending
    """

    def __init__(self, dpath: str=None, show_hidden: bool=False,
                sort_by: str='type', desc: bool=False) -> None:
        # Path is the current working directory if not stated otherwise
        self.path = os.getcwd()
        if dpath: self.path = dpath
        self.show_hidden = show_hidden
        self.sort_by = sort_by
        self.desc = desc

    @staticmethod
    def get_dir_size(start_path: str) -> int:
        """
        Calculates the cumulative size of a given directory.

        Args:
            start_path (str): a path to the folder that's
                    the cumulative file size is calculated.

        Returns:
            int: the size of all files in a given path in bytes
        """
        total_size = 0
        for dirpath, _, filenames in os.walk(start_path):
            for f in filenames:
                fp = os.path.join(dirpath, f)
                total_size += os.path.getsize(fp)
        return total_size

    @classmethod
    def sort_data(cls, data: list, by: str, desc: bool) -> None:
        """
        Sorts data in place, which is inputted as a list, 
        based on a given index(key) and reverses if 
        user has selected descending order.

        Args:
            list: data with several columns
            by: key as a string to sort by
            desc: to sort in descending order
        """
        if by == 'name': column = 0
        elif by == 'dt': column = 1
        elif by == 'size': column = 2
        else: column = -1

        # Sort data inplace based on user's choice
        data.sort(key=lambda x: (x[column], x[-1]), reverse=desc)

    @classmethod
    def _decorate_dir_entry(cls, entry) -> tuple:
        """
        Decorates given entry for a directory. Decorate means that creates 
        a colored representation of a name of the entry, grabs 
        the date it was last modified and size in bytes and decorates.
        collects everything and returns as a list.
        """
        current = []

        # Gives orange color to the string & truncate to 32 chars
        current.append(
            stylize("■ " + entry.name[:33] + "/", fg(202))
        )

        # Get date and convert in to a human readable format
        date = os.stat(entry).st_mtime
        current.append(
            DecoratedData(date, normalize_date('%h %d %Y %H:%M', date))
        ) 

        # recursivly calculates the total size of a folder
        b = DirectoryFiles().get_dir_size(entry)
        current.append(
            DecoratedData(b, bytes_to_human_readable(b))
        )
        
        current.append('-') # Append type 
        return tuple(current)

    @classmethod
    def _decorate_file_entry(cls, entry) -> tuple:
        """
        Decorates given entryfor a file. By decorate it means that creates 
        a colored representation of a name of the entry, grabs 
        the date it was last modified and size in bytes and decorates,
        determins file type. collects everything and returns as a list.
        """
        current = []

        # Gives yellow color to the string & truncate to 32 chars
        current.append(
            stylize("» " + entry.name[:33], fg(226))
        )

        # Convert last modified time (which is in nanoseconds)
        date = os.stat(entry).st_mtime
        current.append(
            DecoratedData(date, normalize_date('%h %d %Y %H:%M', date))
        ) 

        b = os.stat(entry).st_size
        current.append(
            DecoratedData(b, bytes_to_human_readable(b))
        )

        # Evaluate the file type
        current.append(
            magic.from_file(entry.path, mime=True)
        )
        return tuple(current)


    def get_usage(self) -> list:
        """
        Collects the data for a given path like the name of a file/folder 
        and calculates its size if it's a directory, otherwise 
        just grabs a file size. If the current entry in a given 
        path is a file method evaluates its type. Finally, gives 
        us the date when the given file/folder was last modified.

        Returns:
            list: which is a collection of each entry
                (files and folders) in a given path.
        """
        data = []
        with os.scandir(self.path) as it:
            for entry in it:
                try:
                    current = []

                    # Deal with hidden files and folders
                    if entry.name.startswith('.') and not self.show_hidden:
                        continue
                    elif entry.is_file():
                        current = self._decorate_file_entry(entry)
                    elif entry.is_dir():
                        current = self._decorate_dir_entry(entry)

                    # Add current list to the main list
                    if len(current) == 4:
                        data.append(current)
                except FileNotFoundError:
                    continue
        return data

    def tabulate_disk(self) -> tabulate:
        """
        Creates the tabular representation of the data.
        Adds headers and sorts the list's data as rows.

        Returns:
            tabulate: a tabulated form of the current 
                    the directory's folders and files.
        """
        headers = [
            'name',
            'last modified (dt)', 
            'size',
            'type'
        ]
        result = self.get_usage()
        self.sort_data(result, self.sort_by, self.desc)
        return tabulate(result, headers, tablefmt="rst")

    def print_tables(self) -> None:
        """Prints tabular data in the terminal"""
        print(self.tabulate_disk())


if __name__ == '__main__':
    files = DirectoryFiles(sort_by='type', desc=True)
    files.print_tables()